@@ -1,6 +1,7 @@
 context("chemspider")
 token <- '37bf5e57-9091-42f5-9274-650a64398aaf'
 
+require(RCurl)
 chk_cs <- function(){
   qurl <- 'http://www.chemspider.com/Search.asmx/SimpleSearch?query=Triclosan&token=37bf5e57-9091-42f5-9274-650a64398aaf'
   Sys.sleep(0.2)
@@ -14,43 +15,47 @@ chk_cs <- function(){
 test_that("get_csid()", {
   chk_cs()
 
-  expect_equal(get_csid("Triclosan", token = token, verbose = FALSE), '5363')
+  tt <- get_csid("Triclosan", token = token, verbose = FALSE)
+  expect_equal(tt, '5363')
   expect_equal(get_csid("xxxxxxxxx", token = token, verbose = FALSE), NA)
   expect_error(get_csid(c("a", "b"), token = token))
   expect_warning(get_csid(NA, token = token))
-  expect_true(is.vector(get_csid("Triclosan", token = token, verbose = FALSE)))
+  expect_true(is.vector(tt))
 })
 
 
 test_that("cs_compinfo()", {
   chk_cs()
 
-  expect_equal(length(cs_compinfo('5363', token, verbose = FALSE)), 4)
-  expect_equal(cs_compinfo('5363', token, verbose = FALSE)['SMILES'],
-               c(SMILES = "c1cc(c(cc1Cl)O)Oc2ccc(cc2Cl)Cl"))
+  tt <- cs_compinfo('5363', token, verbose = FALSE)
+  expect_equal(length(tt), 5)
+  expect_equal(tt[['smiles']],
+               "c1cc(c(cc1Cl)O)Oc2ccc(cc2Cl)Cl")
   expect_warning(cs_compinfo('aaaa', token, verbose = FALSE))
   expect_error(cs_compinfo(c("a", "b"), token = token))
   })
 
+
 test_that("cs_extcompinfo()", {
   chk_cs()
-
-  expect_equal(cs_extcompinfo('5363', token = token, verbose = FALSE)['AverageMass'],
-               c(AverageMass = '289.5418'))
-  expect_equal(length(cs_extcompinfo('5363', token = token, verbose = FALSE)), 12)
+  tt <- cs_extcompinfo('5363', token = token, verbose = FALSE)
+  expect_equal(tt[['average_mass']],
+               289.5418)
+  expect_equal(length(tt), 13)
   expect_warning(cs_extcompinfo('aaaa', token, verbose = FALSE))
   expect_error(cs_extcompinfo(c("a", "b"), token = token))
 })
 
 
+# integration tests
 test_that("csid_extcompinfo(get_cid())", {
   chk_cs()
-
-  expect_equal(cs_extcompinfo(get_csid('Triclosan', token = token, verbose = FALSE),
-                                token = token, verbose = FALSE)['AverageMass'],
-               c(AverageMass = '289.5418'))
-  expect_equal(length(cs_extcompinfo(get_csid('Triclosan', token = token, verbose = FALSE),
-                                       token = token, verbose = FALSE)), 12)
+  tt <- get_csid('Triclosan', token = token, verbose = FALSE)
+  tt2 <- cs_extcompinfo(tt,
+                        token = token, verbose = FALSE)
+  expect_equal(tt2[['average_mass']],
+               289.5418)
+  expect_equal(length(tt2), 13)
 })
 
 
@@ -76,20 +81,6 @@ test_that("cs_csid_mol()", {
 })
 
 
-test_that("is.inchikey_cs", {
-  expect_message(is.inchikey_cs('BQJCRHHNABKAKU-KBQPJGBKSA-N'))
-  g <- is.inchikey_cs('BQJCRHHNABKAKU-KBQPJGBKSA-N')
-  b <- is.inchikey_cs('BQJCRHHNABKAKU-KBQPJGBKSA')
-
-  expect_true(g)
-  expect_false(b)
-
-  expect_error(is.inchikey_cs(c('BQJCRHHNABKAKU-KBQPJGBKSA', 'BQJCRHHNABKAKU-KBQPJGBKSA-N')))
-
-  expect_equal(length(g), 1)
-})
-
-
 test_that("cs_inchikey_csid()", {
   m1 <- cs_inchikey_csid('BQJCRHHNABKAKU-KBQPJGBKSA-N')
 
@@ -283,4 +274,18 @@ test_that("cs_convert()", {
 
   m9 <- cs_convert(smiles, from = 'smiles', to = 'inchi')
   expect_equal(m9, inchi)
+})
+
+
+test_that("cs_prop()", {
+  id <- '5363'
+  m1 <- cs_prop(id)
+
+  expect_error(cs_prop(c(id, id)))
+
+  expect_is(m1, 'list')
+  expect_equal(length(m1), 3)
+
+  expect_is(m1$epi, 'data.frame')
+
 })
\ No newline at end of file
